/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "user_callbacks.h"
#include "main.h"

#include <data_control.h>

static bool datacontrol_init = false;

static data_control_h provider_map = NULL;
static data_control_h provider_sql = NULL;
static data_control_map_response_cb map_callback;
static data_control_sql_response_cb sql_callback;

static void map_get_response_cb(int request_id, data_control_h provider, char **ret_value_list,
                                int ret_value_count, bool provider_result, const char *error,
                                void *user_data)
{
    if (provider_result) {
        dlog_print(DLOG_INFO, LOG_TAG, "Get operation (id: %d) was successful.", request_id);
        PRINT_MSG("Get operation (id: <b>%d</b>) was successful.", request_id);

        PRINT_MSG("Obtained values:");
        for (int i = 0; i < ret_value_count; i++) {
            dlog_print(DLOG_INFO, LOG_TAG, "%d. value is : %s", i+1, ret_value_list[i]);
            PRINT_MSG("%d. %s", i+1, ret_value_list[i]);
        }
        PRINT_MSG("");
    } else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                   "Get operation (id: %d) failed with error message: %s.", request_id, error);
        PRINT_MSG("Get operation (id: <b>%d</b>) failed with error message: %s.", request_id, error);
    }
}

static void map_set_response_cb(int request_id, data_control_h provider, bool provider_result,
                                const char *error, void *user_data)
{
    PRINT_MSG("Set operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result)
        dlog_print(DLOG_INFO, LOG_TAG, "Set operation (id: %d) was successful.",
                request_id);
    else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Set operation for the request %d failed, error message: %s",
                request_id, error);
    }
}

static void map_add_response_cb(int request_id, data_control_h provider, bool provider_result,
                                const char *error, void *user_data)
{
    PRINT_MSG("Add operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result)
        dlog_print(DLOG_INFO, LOG_TAG, "Add operation (id: %d) was successful.",
                request_id);
    else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Add operation for the request %d failed, error message: %s",
                request_id, error);
    }
}

static void map_remove_response_cb(int request_id, data_control_h provider, bool provider_result,
                                   const char *error, void *user_data)
{
    PRINT_MSG("Remove operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result)
        dlog_print(DLOG_INFO, LOG_TAG, "Remove operation (id: %d) was successful.",
                request_id);
    else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Remove operation for the request %d failed, error message: %s.",
                request_id, error);
    }
}

static void sql_delete_response_cb(int request_id, data_control_h provider, bool provider_result,
                                   const char *error, void *user_data)
{
    PRINT_MSG("Delete operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result)
        dlog_print(DLOG_INFO, LOG_TAG, "Delete operation (id: %d) was successful.",
                request_id);
    else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Delete operation for the request %d failed, error message: %s.",
                request_id, error);
    }
}

static void sql_insert_response_cb(int request_id, data_control_h provider, long long inserted_row_id,
                                   bool provider_result, const char *error, void *user_data)
{
    PRINT_MSG("Insert operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result)
        dlog_print(DLOG_INFO, LOG_TAG, "Insert operation (id: %d) was successful.",
                request_id);
    else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Insert operation for the request %d failed, error message: %s.",
                request_id, error);
    }
}

static void sql_select_response_cb(int request_id, data_control_h provider, result_set_cursor cursor,
                            bool provider_result, const char *error, void *user_data)
{
    PRINT_MSG("Select operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result) {
        dlog_print(DLOG_INFO, LOG_TAG, "Select operation (id: %d) was successful.",
                request_id);

        if (NULL == cursor) {
            dlog_print(DLOG_INFO, LOG_TAG, "The SQL result set is NULL.");
            PRINT_MSG("No result matching the query found.<br>");
            return;
        }

        int result = data_control_sql_step_first(cursor);
        if (DATA_CONTROL_ERROR_NONE != result) {
            DLOG_PRINT_ERROR("data_control_sql_step_first()", result);
            PRINT_MSG("Could not set the cursor to the first found element.<br>");
            return;
        }

        do {
            char word[32] = { 0, };
            char word_desc[32] = { 0, };

            result = data_control_sql_get_text_data(cursor, 0, word);
            if (DATA_CONTROL_ERROR_NONE != result) {
                DLOG_PRINT_ERROR("data_control_sql_get_text_data()", result);
                PRINT_MSG("Could not get the text.");
            } else {
                dlog_print(DLOG_INFO, LOG_TAG, "Word : %s", word);
                PRINT_MSG("Word : <b>%s</b>", word);
            }


            result = data_control_sql_get_text_data(cursor, 1, word_desc);
            if (DATA_CONTROL_ERROR_NONE != result) {
                DLOG_PRINT_ERROR("data_control_sql_get_text_data()", result);
                PRINT_MSG("Could not get the text.");
            } else {
                dlog_print(DLOG_INFO, LOG_TAG, "Word DESC : %s", word_desc);
                PRINT_MSG("Word DESC : <b>%s</b><br>", word_desc);
            }
        } while (DATA_CONTROL_ERROR_NONE == data_control_sql_step_next(cursor));
    } else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Select operation for the request %d failed, error message: %s.",
                request_id, error);
    }
}

static void sql_update_response_cb(int request_id, data_control_h provider, bool provider_result,
                                   const char *error, void *user_data)
{
    PRINT_MSG("Update operation (id: <b>%d</b>) %s.<br>",
            request_id, provider_result ? "successful" : "failed");

    if (provider_result)
        dlog_print(DLOG_INFO, LOG_TAG, "Update operation (id: %d) was successful.",
                request_id);
    else {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Update operation for the request %d failed, error message: %s.",
                request_id, error);
    }
}

void initialize_datacontrol_consumer(appdata_s *ad)
{
    if (datacontrol_init)
        return;

    /*
     * Working with Map-type Data Controls
     */

    // Create a Map-type data control handle.
    int error_code = data_control_map_create(&provider_map);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_map_create()", error_code);
        PRINT_MSG("Could not create map-type data control.");
        return;
    }

    // Set map provider id.
    const char *map_provider_id = elm_entry_entry_get(ad->data_provider_map_provider_id_entry);

    error_code = data_control_map_set_provider_id(provider_map, map_provider_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_map_set_provider_id()", error_code);
        PRINT_MSG("Could not set map provider id.");
        return;
    }

    // Set map data ID.
    const char *map_data_id = "table";

    error_code = data_control_map_set_data_id(provider_map, map_data_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_map_set_data_id()", error_code);
        PRINT_MSG("Could not set map data id.");
        return;
    }

    // Set map provider response callbacks.
    map_callback.get_cb = map_get_response_cb;
    map_callback.set_cb = map_set_response_cb;
    map_callback.add_cb = map_add_response_cb;
    map_callback.remove_cb = map_remove_response_cb;

    // Register map provider response callback.
    error_code = data_control_map_register_response_cb(provider_map, &map_callback, NULL);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_map_register_response_cb()", error_code);
        PRINT_MSG("Could not register map provider response callbacks.");
        return;
    }

    /*
     *  Working with SQL-type Data Controls
     */

    // Create a SQL-type data control handle.
    error_code = data_control_sql_create(&provider_sql);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_create()", error_code);
        PRINT_MSG("Could not create SQL-type data control.");
        return;
    }

    // Set SQL provider id.
    const char *sql_provider_id = elm_entry_entry_get(ad->data_provider_sql_provider_id_entry);

    error_code = data_control_sql_set_provider_id(provider_sql, sql_provider_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_set_provider_id()", error_code);
        PRINT_MSG("Could not set SQL provider id.");
        return;
    }

    // Set SQL data ID.
    const char *sql_data_id = "Dictionary";

    error_code = data_control_sql_set_data_id(provider_sql, sql_data_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_set_data_id()", error_code);
        PRINT_MSG("Could not set SQL data id.");
        return;
    }

    // Set SQL provider response callbacks.
    sql_callback.delete_cb = sql_delete_response_cb;
    sql_callback.insert_cb = sql_insert_response_cb;
    sql_callback.select_cb = sql_select_response_cb;
    sql_callback.update_cb = sql_update_response_cb;

    // Register SQL provider response callback.
    error_code = data_control_sql_register_response_cb(provider_sql, &sql_callback, NULL);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_register_response_cb()", error_code);
        PRINT_MSG("Could not register SQL provider response callbacks.");
    }

    datacontrol_init = true;
}

static void _map_data_control(appdata_s *ad, Evas_Object *obj, void *event_info)
{
    /*
     * Use data Map-type Data Controls
     */

    int req_id = 0;

    // Add new value for the key to the data control map.
    const char *key = "key";
    const char *value = "value";

    int error_code = data_control_map_add(provider_map, key, value, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Adding %s-%s pair failed with error: %d.",
                key, value, error_code);
        PRINT_MSG("Could not add value: <b>%s</b> for the key: <b>%s</b>.<br>",
                value, key);
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for adding %s-%s pair is: %d.",
                key, value, req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for adding value: <b>%s</b> for the key: <b>%s</b> "
                "was successfully sent.<br>", req_id, value, key);
    }

    // Get the values list associated with the given key from data control map.
    error_code = data_control_map_get(provider_map, key, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        dlog_print(DLOG_ERROR, LOG_TAG,
                   "Getting the values list of the key: %s failed with error: %d.", key, error_code);
        PRINT_MSG("Could not get the values list of the key: <b>%s</b>.<br>", key);
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for getting values list of the key: %s is: %d.",
                key, req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for obtaining values list of the key: <b>%s</b> "
                "was successfully sent.<br>", req_id, key);
    }

    // Set new value for the key.
    const char *old_value = "old value";
    const char *new_value = "new value";

    error_code = data_control_map_set(provider_map, key, old_value, new_value, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        dlog_print(DLOG_ERROR, LOG_TAG,
                "Replacing old_value: %s with new_value: %s failed with error: %d.",
                old_value, new_value, error_code);
        PRINT_MSG("Could not change the old value: <b>%s</b> to new value: <b>%s</b> for the key: <b>%s</b>.<br>",
                old_value, new_value, key);
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for changing value: %s of the key: %s to: %s is: %d.",
                old_value, key, new_value, req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for changing value: <b>%s</b> of the key: <b>%s</b> to <b>%s</b> "
                "was successfully sent.<br>", req_id, old_value, key, new_value);
    }

    // Get the current value of the given key from data control map.
    error_code = data_control_map_get(provider_map, key, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        dlog_print(DLOG_ERROR, LOG_TAG,
                   "Getting the value of the key: %s failed with error: %d.", key, error_code);
        PRINT_MSG("Could not get the value of the key: <b>%s</b> from the data control map.<br>", key);
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for getting key: %s values is: %d.", key, req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for obtaining values list of the key: <b>%s</b> "
                "was successfully sent.<br>", req_id, key);
    }

    // Remove the key-value pair from the data control map.
    error_code = data_control_map_remove(provider_map, key, value, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Removing %s-%s pair failed with error: %d.",
                key, value, error_code);
        PRINT_MSG("Could not remove the key: <b>%s</b> value: <b>%s</b> pair from the data control map.<br>",
                key, value);
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for removing value: %s from the key: %s is: %d.",
                value, key, req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for removing value: <b>%s</b> from the key: <b>%s</b> "
                "was successfully sent.<br>", req_id, value, key);
    }
}

static void _sql_data_control(appdata_s *ad, Evas_Object *obj, void *event_info)
{
    /*
     * Use data SQL-type Data Controls
     */

    int req_id = 0;

    // Insert new row to the table owned by the SQL-type data control provider.
    bundle *b = bundle_create();
    int error_code = get_last_result();
    if (BUNDLE_ERROR_NONE != error_code || NULL == b) {
        DLOG_PRINT_ERROR("bundle_create()", error_code);
        PRINT_MSG("Could not create a bundle.");
        return;
    }

    const char *key1 = "WORD";
    const char *value1 = "'test'";

    error_code = bundle_add_str(b, key1, value1);
    if (BUNDLE_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("bundle_add_str()", error_code);
        PRINT_MSG("Could not add %s-%s pair to the bundle.", key1, value1);
    }

    const char *key2 = "WORD_DESC";
    const char *value2 = "'test desc'";

    error_code = bundle_add_str(b, key2, value2);
    if (BUNDLE_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("bundle_add_str()", error_code);
        PRINT_MSG("Could not add %s-%s pair to the bundle.", key2, value2);
    }

    error_code = data_control_sql_insert(provider_sql, b, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_insert()", error_code);
        PRINT_MSG("Could not insert new row.");
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for inserting new row is: %d.", req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for inserting new row was successfully sent.<br>",
                req_id);
        PRINT_MSG("Row to be inserted:<br>"
                "column: <b>%s</b> value: <b>%s</b><br>"
                "column: <b>%s</b> value: <b>%s</b>",
                key1, value1, key2, value2);
    }

    PRINT_MSG("<br>-----<br>");

    // Select rows.
    char *column_list[2];
    column_list[0] = (char *)key1;
    column_list[1] = (char *)key2;

    const char *where = "WORD = 'test'";
    const char *order = "WORD ASC";

    error_code = data_control_sql_select(provider_sql, column_list, 2, where, order, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_select()", error_code);
        PRINT_MSG("Could not query for rows.");
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for querying rows is: %d.", req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for querying rows was successfully sent.<br>",
                req_id);
        PRINT_MSG("Query:<br>WHERE <b>%s</b><br>ORDER BY <b>%s</b>", where, order);
    }

    PRINT_MSG("<br>-----<br>");

    // Update rows.
    error_code = bundle_del(b, key1);
    if (BUNDLE_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("bundle_del()", error_code);
        PRINT_MSG("Could not remove <b>%s</b> key from the bundle.", key1);
        return;
    }

    const char *value3 = "'test_new'";
    error_code = bundle_add_str(b, key1, value3);
    if (BUNDLE_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("bundle_add_str()", error_code);
        PRINT_MSG("Could not add <b>%s-%s</b> pair to the bundle.", key1, value3);
    }

    error_code = data_control_sql_update(provider_sql, b, where, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_update()", error_code);
        PRINT_MSG("Could not update rows.");
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for updating rows is: %d.", req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for updating rows was successfully sent.<br>",
                req_id);
        PRINT_MSG("Value: <b>%s</b> of the key: <b>%s</b> will be updated to: <b>%s</b>",
                value1, key1, value3);
    }

    error_code = bundle_free(b);
    if (BUNDLE_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("bundle_free()", error_code);
        PRINT_MSG("Could not free the bundle.");
    }

    PRINT_MSG("<br>-----<br>");

    // Select rows.
    error_code = data_control_sql_select(provider_sql, column_list, 2, where, order, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_select()", error_code);
        PRINT_MSG("Could not query for rows.");
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for querying rows is: %d", req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for querying rows was successfully sent.<br>",
                req_id);
        PRINT_MSG("Query:<br>WHERE <b>%s</b><br>ORDER BY <b>%s</b>", where, order);
    }

    PRINT_MSG("<br>-----<br>");

    // Delete rows.
    const char *where_delete = "WORD = 'test'";

    error_code = data_control_sql_delete(provider_sql, where_delete, &req_id);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_delete()", error_code);
        PRINT_MSG("Could not delete rows.");
    } else {
        dlog_print(DLOG_INFO, LOG_TAG, "Request id for deleting rows is: %d", req_id);
        PRINT_MSG("Request (id: <b>%d</b>) for deleting rows was successfully sent.<br>",
                req_id);
        PRINT_MSG("Deletion query:<br>WHERE <b>%s</b>", where_delete);
    }

    PRINT_MSG("<br>-----<br>");
}

void deinitialize_datacontrol_consumer()
{
    if (!datacontrol_init)
        return;

    int error_code = data_control_map_unregister_response_cb(provider_sql);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_map_unregister_response_cb()", error_code);
        PRINT_MSG("Could not unregister map provider response callback.");
    }

    error_code = data_control_map_destroy(provider_map);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_map_destroy()", error_code);
        PRINT_MSG("Could not destroy map provider.");
    }

    error_code = data_control_sql_unregister_response_cb(provider_sql);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_unregister_response_cb()", error_code);
        PRINT_MSG("Could not unregister SQL provider response callback.");
    }

    error_code = data_control_sql_destroy(provider_sql);
    if (DATA_CONTROL_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("data_control_sql_destroy()", error_code);
        PRINT_MSG("Could not destroy SQL provider.");
    }

    datacontrol_init = false;
}

void create_buttons_in_main_window(appdata_s *ad)
{
    Evas_Object *display = _create_new_cd_display(ad, "Data Control Consumer", NULL);

    _new_button(ad, display, "Working with Map-type Data Control", _map_data_control);
    _new_button(ad, display, "Working with SQL-type Data Control", _sql_data_control);
}
